"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts_simple_ast_1 = require("ts-simple-ast");
// Statement&ReferenceFindableNode
function moveNode(node, destFile) {
    const nodeFile = node.getSourceFile();
    const references = getReferences(node);
    const nodeFileImports = nodeFile.getImportDeclarations();
    // we copy all the imports from nodeFile to destFile and the organize imports so the moved declaration don't miss any import
    // TODO: class declaration maybe using references to nodes inside destFile non exported or imported
    destFile.addImportDeclarations(nodeFileImports.filter(i => i.getModuleSpecifierSourceFile() !== destFile).map(i => {
        return Object.assign({}, i.getStructure(), { moduleSpecifier: destFile.getRelativePathAsModuleSpecifierTo(i.getModuleSpecifierSourceFile()) });
    }));
    // For each referenced sourceFile  - each import that references the nodeFile we duplicate them fixing the target path to dest.
    const referencedSourceFiles = references.map(r => r.getSourceFile()).filter((f, i, a) => a.indexOf(f) === i && f !== nodeFile);
    referencedSourceFiles.forEach(f => {
        const newImports = f.getImportDeclarations().filter(i => i.getModuleSpecifierSourceFile() === nodeFile).map(i => (Object.assign({}, i.getStructure(), { moduleSpecifier: f.getRelativePathAsModuleSpecifierTo(destFile) })));
        f.addImportDeclarations(newImports);
        f.getImportDeclarations().filter(i => i.getModuleSpecifierSourceFile() === nodeFile).forEach(i => i.remove());
    });
    // move the declaration
    if (ts_simple_ast_1.TypeGuards.isClassDeclaration(node)) {
        destFile.addClass(node.getStructure());
    }
    else if (ts_simple_ast_1.TypeGuards.isInterfaceDeclaration(node)) {
        destFile.addInterface(node.getStructure());
    }
    else if (ts_simple_ast_1.TypeGuards.isFunctionDeclaration(node)) {
        destFile.addFunction(node.getStructure());
    }
    node.remove();
    destFile.organizeImports();
    nodeFile.organizeImports();
}
exports.moveNode = moveNode;
function getReferences(node) {
    const references = [];
    const referencedSymbols = node.findReferences();
    for (const referencedSymbol of referencedSymbols) {
        for (const reference of referencedSymbol.getReferences()) {
            references.push(reference);
        }
    }
    return references;
}
//# sourceMappingURL=moveNode.js.map